@@ -7,6 +7,7 @@ import logging
 import signal
 import sys
 
+from rejected import common
 from rejected import mcp
 from rejected import __version__
 
@@ -41,6 +42,7 @@ class Controller(clihelper.Controller):
     def _setup(self):
         """Continue the run process blocking on MasterControlProgram.run"""
         # If the app was invoked to specified to prepend the path, do so now
+        common.add_null_handler()
         if self._options.prepend_path:
             self._prepend_python_path(self._options.prepend_path)
 
@@ -82,7 +84,7 @@ class Controller(clihelper.Controller):
             LOGGER.info('Caught CTRL-C, shutting down')
             clihelper.setup_logging(self._debug)
         if self.is_running:
-            self._shutdown()
+            self.stop()
 
 
 def _cli_options(parser):
@@ -109,7 +111,8 @@ def _cli_options(parser):
                       help='Prepend the python path with the value.')
     parser.add_option('-q', '--qty',
                       action='store',
-                      default=None,
+                      type='int',
+                      default=1,
                       dest='quantity',
                       help='Run the specified quanty of consumer processes '
                            'when used in conjunction with -o')
